# coding: utf-8
from __future__ import unicode_literals

import io
import os
import re
import six
import socket
import subprocess
import click
from jmb.tools.functions import execute, execute0

def get_config_filename():
    return os.path.join(click.get_app_dir('jmb'), 'jmb.ini')

def read_config_file(cfg=None):
    if not cfg:
        cfg = get_config_filename()
    parser = six.moves.configparser.SafeConfigParser()
    parser.read([cfg])
    return parser

def get_os():
    "Return a dict with os, rel, name"

    ret_code, out, err = execute('lsb_release -a')
    info = {}
    for line in out.splitlines():
        line = line.decode('utf-8')
        if line.startswith('Distributor ID'):
            info['distributor'] = line.split('\t')[1]
        elif line.startswith('Release'):
            info['release'] = line.split('\t')[1]
        elif line.startswith('Codename'):
            info['codename'] = line.split('\t')[1]
    return info

def get_ip():
    s = socket.socket()
    s.connect(('google.com', 80))
    return s.getsockname()[0]

def get_hostname():
    return socket.gethostname()

def get_keys():
    ret_code, out, err = execute('add-key')
    keys = out.decode('utf-8')
    return [line.split()[-1] for line in keys.splitlines()]

def get_project(directory='.'):
    info = {}
    if not os.path.exists(os.path.join(directory, 'setup.py')):
        return info
    info['name'] = os.path.basename(os.path.abspath(directory))
    #info['packages'] = get_packages(directory)
    cp = six.moves.configparser.ConfigParser()
    cp.read(os.path.join(os.path.abspath(directory), '.hg/hgrc'))
    try:
        info['repo'] = cp.get('paths', 'default')
    except:
        info['repo'] = None
    ret_code, out, err = execute('jmb-test-setup')

    return info

def get_pyversion(directory='.'):
    ret_code, out, err = execute0('head -n 1 %s/bin/buildout' % directory)
    return '3' if 'python3' in out.decode('utf-8') else '2'

def get_buildout_version(directory='.'):
    "return a tuple major release and complete rel"
    ret_code, out, err = execute0("grep zc.buildout.*egg %s/bin/buildout" % directory)
    rel = out.decode('utf-8').split('-')[1]
    return rel[0], rel

def get_packages(directory='.'):
    cwd = os.getcwd()
    os.chdir(directory)
    ret_code, out, err = execute("jmb-test-setup")
    packages = {}
    for line in out.decode('utf-8').splitlines():
        m = re.search('/(?P<name>\w+)-(?P<version>[\d\.\w]+)-py.*.egg', line)
        if m:
            packages[m.groupdict()['name']] = (m.groupdict()['version'], True)
        elif '.src' in line:
            pkg = line.split('/')[-1]
            packages[pkg] = ('source', False)

    os.chdir(cwd)
    return packages

def get_debug(directory="."):
    ret_code, out, err = execute('bin/django diffsettings')
    for line in out.decode('utf-8').splitlines():
        if line.startswith('DEBUG ='):
            return line.split(' = ')[1]

def get_settings_link(directory='.'):
    settings_file = os.path.join(directory, 'web/settings/settings.py')
    return os.readlink(settings_file).split('.')[0]

def get_celery(directory='.'):
    nick_file = os.path.join(directory, 'bin/nick')
    if not os.path.exists(nick_file):
        return False
    with io.open(nick_file) as f:
        nick = f.readline().split()[0]
    ini_file = '/etc/uwsgi-emperor/vassals-enabled/{nick}.ini'.format(nick=nick)
    ini_file2 = '/etc/uwsgi/{nick}.ini'.format(nick=nick)
    if os.path.exists(ini_file):
        pass
    elif os.path.exists(ini_file2):
        ini_file = ini_file2
    else:
        return None
    cp = si.moves.configparser.ConfigParser()
    cp.read(ini_file)
    return cp.get('uwsgi', 'attach_daemon2') and True or False

def get_server_info():
    data = get_os()
    data['ip'] = get_ip()
    data['name'] = get_hostname()
    data['keys'] = get_keys()
    return data

def get_project_info(directory=".", test=False):
    data = get_project(directory)
    data['settings'] = get_settings_link()
    data['server'] = get_hostname()
    data['py_version'] = get_pyversion()
    data['buildout'] = get_buildout_version(directory)[0]
    data['debug'] = get_debug(directory)
    data['test'] = test
    data['celery'] = get_celery()
    return data
